"""Gestión de los parámetros del CLI."""

import argparse

def get_cli_parameters(resources):
    """Gestión de parámetros del CLI."""
    loc = resources['loc']
    parser = argparse.ArgumentParser(description=loc['menu']['descrip'])

    parser.add_argument('-p', '--path_repo', type=str,
                        help=loc['menu']['p'], required=True)
    parser.add_argument('-f', '--file_out', type=str,
                        help=loc['menu']['f'], required=True)
    parser.add_argument('-o', '--output_format',
                        type=str, choices=['csv', 'excel'],
                        default='csv', required=False, help=loc['menu']['o'])

    args = parser.parse_args()
    return args.path_repo, args.file_out, args.output_format



"""Analiza los ficheros de parámetros de los accionamientos NIDEC."""

import logging
from pathlib import Path

from helpers.file_helpers import FileUtilities
from nidec.nidec_param_file_engine import NidecParamFileEngine

logger = logging.getLogger(__name__)


def _analysis(parameter_file, resources):
    """Analyze the parameter file and return the header information.

    :param parameter_file: The file path to the parameter file that contains
        the configuration or data to be analyzed.
    :resources: The configuration and localization resources, otherwise None
        is returned.
    :return: The header information extracted from the Nidec parameter file,
        or None if the analysis fails.

    # TODO: td_ref_02
    # TODO: td_ref_03
    # TODO: td_ref_04
    # TODO: td_ref_05
    # TODO: td_ref_06
    """
    config = resources['config']
    loc = resources['loc']

    # ETIQUETA RAIZ
    etiqueta_raiz = config['config']['root_tag']

    try:
        # Datos del accionamiento extraidos de su fichero de parámetros
        nidec_drive = NidecParamFileEngine(
            parameter_file, etiqueta_raiz, resources)
        return nidec_drive.drive_header()
    except FileNotFoundError:
        logger.error(loc['err']['e1'], parameter_file)
        return None
    except Exception as e:
        logger.error(loc['err']['e3'], parameter_file, e)
        return None


def inventory_generator(path_repo, file_out, out_type, resources):
    """Genera el inventario de accionamientos.

    :param path_repo: Path del repositorio.
    :param file_out: Nombre del fichero de salida, sin extensión.
    :param out_type: Tipo de fichero de salida: csv o excel
    :param resources: configuración y localización
    :return True if the process concludes successfully, otherwise False.
    """
    config = resources['config']
    loc = resources['loc']

    # NOMBRES DE FICHEROS EXCLUIDOS
    files_excluded = config['config']['excluded_files']
    # EXTENSIÓN FICHERO PARÁMETROS
    ext_par_file = f"*{config['config']['ext_par_file']}"

    header_container = []

    try:
        parameter_repo = Path(path_repo).resolve()
        parameter_files = FileUtilities.recursive_file_listing(
            parameter_repo, ext_par_file, files_excluded)
    except FileNotFoundError:
        logger.error(loc['err']['e11'])
        return False

    # Acceso a cada fichero del repositorio
    try:
        for parameter_file in parameter_files:
            # Extraer información del fichero
            nidec_drive_header = _analysis(parameter_file, resources)
            # Si se ha extraido información del accionamiento, almacenarla
            if nidec_drive_header is not None:
                header_container.append(nidec_drive_header)
    except FileNotFoundError:
        return False

    # Generar fichero de salida con los datos
    if _output_data_file_generator(
            file_out, out_type, header_container, resources):
        logger.info(loc['info']['i1'])
        return True

    logger.error(loc['err']['e10'])
    return False


def _output_data_file_generator(file_out, out_type, header_container,
                                resources):
    """Generador de fichero de datos.

    Permite seleccionar el tipo de fichero de salida, pudiendo ser este
    de tipo excel o csv.

    :param file_out: Nombre del fichero de salida de datos
    :param out_type: Tipo de fichero de salida: csv o excel
    :param header_container: Contenedor de datos a guardar en el fichero.
    :return True if the process concludes successfully, otherwise False.
    """
    loc = resources['loc']

    try:
        results_file = Path(file_out).resolve()
    except FileNotFoundError:
        logger.error(loc['err']['e11'])
        return False

    try:
        if out_type in ('excel', 'csv'):
            FileUtilities.data_out_file(
                results_file, header_container, out_type)
            return True

        logger.error(loc['err']['e2'], out_type)
        return False
    except Exception as e:
        logger.error(loc['err']['e4'], e)
        return False



"""Conjunto de métodos destinados a la gestión del sistema de ficheros.

NOTA: Solo situar utilidades de ficheros de uso genérico!!
"""

import logging
from pathlib import Path
import pandas as pd
import yaml

logger = logging.getLogger(__name__)

MSG_1 = '[INFO]: limpieza de directorios.'
MSG_2 = '[INFO]: generando lista de ficheros.'
MSG_3 = '[ERROR]: imposible acceder al path especificado.'
MSG_4 = '[ERROR]: inesperado: %s'
MSG_5 = '[INFO]: limpieza finalizada.'
MSG_6 = '[INFO]: limpieza de ficheros por extensión.]'
MSG_7 = '[PTH->]: %s'
MSG_8 = '[EXT->]: %s'
MSG_9 = '[INFO]: limpieza de todos los ficheros.'
MSG_10 = '[ERROR]: fichero %s inaccesible. %s'
MSG_11 = '[ERROR]: fichero %s inadecuado.'
MSG_12 = '[ERROR]: %s al convertir los datos a DataFrame para el archivo %s.'
MSG_13 = '[ERROR]: formato %s no válido. Debe ser csv o excel'
MSG_14 = '[INFO]: Archivo %s generado correctamente.'
MSG_15 = '[ERROR]: %s al intentar guardar el archivo %s.'
MSG_16 = '[ERROR]: %s al generar el archivo %s.'

class FileUtilities:
    """Utilidades de gestión del sistema de ficheros."""

    @staticmethod
    def recursive_file_listing(path, extension, files_excluded):
        """Listado recursivo de ficheros con una extensión dada.

        Se parte de un path dado y se excluyen aquellos ficheros cuyo
        nombre está en la lista file_excluded.

        :param path: Path del repositorio.
        :param extension: Extensión de los ficheros a listar.
        :param files_excluded: Lista de ficheros a excluir.
        :return: Lista de ficheros.
        """
        path = Path(path)
        files = []

        try:
            if path.exists():
                logger.info(MSG_7, path)
                logger.info(MSG_8, extension)
                for child in path.rglob(extension):
                    if child.is_file():
                        if child.stem not in files_excluded:
                            files.append(child)
                logger.info(MSG_2)
            else:
                logger.error(MSG_3)
        except Exception as e:
            logger.error(MSG_4, e)
        return files


    @staticmethod
    def recursive_file_listing_test(path, extension, files_excluded):
        """Listado recursivo de ficheros con una extensión dada.

        Se parte de un path dado y se excluyen aquellos ficheros cuyo
        nombre está en la lista file_excluded.

        :param path: Path del repositorio.
        :param extension: Extensión de los ficheros a listar.
        :param files_excluded: Lista de ficheros a excluir.
        :return: Lista de ficheros.
        """
        files = []
        if path.exists():
            logger.info(MSG_7, path)
            logger.info(MSG_8, extension)
            for child in path.rglob(extension):
                if child.is_file():
                    if child.stem not in files_excluded:
                        files.append(child)
                        # TODO: td_ref_01
            logger.info(MSG_2)
        else:
            logger.info(MSG_3)
        return files


    @staticmethod
    def remove_file_if_extension_not_equal(path, extension):
        """Elimina los ficheros cuya extensión no coincida con la dada.

        :param path: Path del repositorio.
        :param extension: Extensión de los ficheros a listar.
        """
        if path.exists():
            logger.info(MSG_7, path)
            logger.info(MSG_6)
            for child in path.rglob('*.*'):
                if child.is_file():
                    if child.suffix not in extension or child.suffix == '':
                        child.unlink()
            logger.info(MSG_5)
        else:
            logger.info(MSG_3)


    @staticmethod
    def remove_directory_if_name_in_list(path, removable_directories):
        """Elimina los directorios si su nombre están en la lista.

        :param path: Path del repositorio.
        :param removable_directories: Directorios a eliminar.
        """
        if path.exists():
            logger.info(MSG_7, path)
            logger.info(MSG_1)
            for child in path.rglob('**/*'):
                if child.is_dir():
                    if child.name in removable_directories:
                        logger.info(child)
                        child.rmdir()
            logger.info(MSG_5)
        else:
            logger.error(MSG_3)


    @staticmethod
    def remove_all_files(path):
        """Elimina todos los ficheros del path.

        :param path: Path a limpiar.
        """
        if path.exists():
            logger.info(MSG_7, path)
            logger.info(MSG_9)
            for child in path.rglob('*.*'):
                if child.is_file():
                    child.unlink()
            logger.info(MSG_5)
        else:
            logger.info(MSG_3)


    @staticmethod
    def read_yaml(yaml_file):
        """Load the yaml file and return the contents as a dictionary.

        :param yaml_file: Path to the yaml file.
        :return: Dictionary with the contents of the yaml file or None
            if the file is not accessible.
        """
        try:
            with open(yaml_file, 'r', encoding='utf8') as f:
                return yaml.safe_load(f)
        except FileNotFoundError as e:
            logger.error(MSG_10, yaml_file, e)
            return None
        except yaml.YAMLError:
            logger.error(MSG_11, yaml_file)
            return None


    @staticmethod
    def data_out_file(output_filename, data, output_format):
        """Generate data file, in csv or excel, from the provided data.

        :param output_filename: Name of the output file (without extension).
        :param data: Data to be exported to CSV/Excel format.
        :param output_format: Format of the output file. Can be 'csv' or 'excel'.
        :return: True if the file is generated successfully, False otherwise.
        """
        try:
            df = pd.DataFrame(data)
        except (TypeError, pd.errors.ParserError) as e:
            logger.error(MSG_12, e, output_filename)
            raise

        if output_format == 'csv':
            file_path = Path(f"{output_filename}.csv").resolve()
        elif output_format == 'excel':
            file_path = Path(f"{output_filename}.xlsx").resolve()
        else:
            logger.error(MSG_13, output_format)
            return False

        try:
            if output_format == 'csv':
                df.to_csv(file_path, index=False)
            elif output_format == 'excel':
                df.to_excel(file_path, index=False)
            else:
                logger.error(MSG_13, output_format)
                return False
        except Exception as e:
            logger.error(MSG_16, e, output_filename)
            raise

        logger.info(MSG_14, file_path)
        return True



"""Módulo principal, gestiona los parámetros de entrada."""

import os
import sys
from pathlib import Path

from cli_parameters import get_cli_parameters
from helpers.file_helpers import FileUtilities
from nidec.drive_inventory import inventory_generator

root_path = Path(os.path.dirname(__file__))

MSG_0 = '[ERROR]: No se ha podido ejecutar el programa o ha ocurrido un error.'
MSG_1 = '[INFO]: Proceso concluido.'

# ARCHIVO DE LOCALIZACIÓN
LOC_FILE = Path(f"{root_path}/resources/es.yaml")
loc = FileUtilities.read_yaml(LOC_FILE)

# ARCHIVO DE CONFIGURACIÓN
CONFIG_FILE = Path(f"{root_path}/resources/config.yaml")
config = FileUtilities.read_yaml(CONFIG_FILE)

resources = {'loc': loc, 'config': config}


def main():
    """Función principal."""
    path_repo, file_out, out_type = get_cli_parameters(resources)
    return inventory_generator(path_repo, file_out, out_type, resources)


if __name__ == '__main__':
    if loc is not None and config is not None:
        if main():
            print(MSG_1)
        else:
            print(MSG_0)
            sys.exit(1)
    else:
        print(MSG_0)
        sys.exit(1)



"""Motor de análisis para ficheros de parámetros de accionamientos NIDEC."""

import logging
import xml.etree.ElementTree as ET

from nidec.nidec_drive import NidecDrive

logger = logging.getLogger(__name__)

"""
Componentes de un motor de análisis
1. Entrada de Datos:
    Los datos pueden venir en diferentes formas: listas, archivos, strings,
    estructuras como DataFrames, o incluso flujos en tiempo real.
2. Procesador:
    Módulo principal que aplica reglas de análisis, algoritmos estadísticos,
    métodos de minería de datos o procesamiento lógico textual.
3. Resultado:
    Salida procesada: métricas consolidadas, patrones detectados o
    incluso visualizaciones.
"""

# ARCHIVO DE LOCALIZACIÓN
# LOC_FILE = 'es.yaml'
# loc = FileUtilities.read_yaml(LOC_FILE)
# loc = self.resources['loc']


class NidecParamFileEngine:
    """Análisis de ficheros de parametrización de los variadores Nidec.

    Proporciona métodos que permiten la extracción de las principales
    características de los variadores del repositorio así como realizar
    búsquedas de un parámetro.

    TODO: td_ref_07
    TODO: td_ref_08
    TODO: td_ref_09
    TODO: td_ref_10
    TODO: td_ref_11
    """

    def __init__(self, param_file, root_tag, resources):
        """Class constructor."""
        self.param_file = param_file
        self.root_tag = root_tag
        self.root_val = self._xml_root()
        self.resources = resources
        self.loc = resources['loc']

    def _xml_root(self):
        """Si root_tag es adecuada, retorna la estructura xml del fichero.

        Sobre la root_tag: es la primera etiqueta del fichero de parámetros,
        normalmente suele ser <ParameterFile>.

        :return: The root of the XML structure of the parameter file, or None
        """
        try:
            tree = ET.parse(self.param_file)
            root = tree.getroot()
            if root.tag == self.root_tag:
                return root
        except (FileNotFoundError, ET.ParseError) as e:
            logger.error(self.loc['err']['e5'], e)

        return None

    def _param_format(self, param):
        """Verifica el formato del número de parámetro.

        Verifica que el número de parámetro pasado como parámetro emplea el
        formato adecuado, de lo contrario lo adapta. Si no es posible adaptar
        el número de parámetro retorna None.

        Verificaciones realizadas:
        - El separador tiene que ser el punto `.`

        Adaptaciones realizadas:
        - Elimina los ceros iniciales: 01.00001 debería de tener el formato
        1.1 salvo para el 1.0

        TODO: td_ref_12
        TODO: td_ref_13
        TODO: td_ref_14
        """
        param = param.replace(',', '.')  # Corregir separador
        if '.' not in param:
            logger.error(self.loc['err']['e6'], param)
            return None

        # Dividir en partes
        param_parts = param.split('.')
        param_part_1 = param_parts[0].strip().lstrip('0') or '0'
        param_part_2 = param_parts[1].strip().lstrip('0') or '0'

        # Casos especiales, evita caso '1.0'
        if param_part_1 == '1' and param_part_2 == '0':
            return '1.0'

        return f'{param_part_1}.{param_part_2}'

    def _scan_slots(self, nidec_drive):
        """Identifica si las cartas opción instaladas en el variador."""
        if self.root_val is None:
            logger.error(self.loc['err']['e7'])
            return None

        try:
            for p in self.root_val.iter('option'):
                slot = p.attrib.get('slot')
                if slot == '1':
                    for n in p.iter('DeviceName'):
                        nidec_drive.slot1 = n.text
                elif slot == '2':
                    for n in p.iter('DeviceName'):
                        nidec_drive.slot2 = n.text
                elif slot == '3':
                    for n in p.iter('DeviceName'):
                        nidec_drive.slot3 = n.text
                elif slot == '4':
                    for n in p.iter('DeviceName'):
                        nidec_drive.slot4 = n.text
                else:
                    return None
        except KeyError:
            return '[SLOT] ERROR/NO DISPONIBLE'

    def _ip_node(self, nidec_drive):
        """Dirección IP / Nodo del variador."""
        if self.root_val is None:
            logger.error(self.loc['err']['e7'])
            return None

        try:
            for p in self.root_val.iter('Connection'):
                nidec_drive.ip_nodo = p.attrib['Address']
        except KeyError:
            return '[IP/NODO] ERROR/NO DISPONIBLE'

    def _modo_trabajo(self, nidec_drive):
        """Modo de trabajo definido en el variador."""
        if self.root_val is None:
            logger.error(self.loc['err']['e7'])
            return None

        # Utiliza ruta xml
        try:
            for p in self.root_val.findall("./drive/Configuration/Mode"):
                nidec_drive.mode = p.text
        except KeyError:
            return '[MODO] ERROR/NO DISPONIBLE'

    @staticmethod
    def _name(tag):
        """Extrae el nombre del variador.

        TODO: td_ref_15
        """
        try:
            return tag.attrib['name']
        except KeyError:
            return '[NAME] ERROR/NO DISPONIBLE'

    @staticmethod
    def _serial_number(tag):
        """Extrae el número de serie del variador.

        TODO: td_ref_16
        """
        try:
            return tag.attrib['serialnumber']
        except KeyError:
            return '[SERIAL NUMBER] ERROR'

    @staticmethod
    def _firmware(tag):
        """Extrae el número de firmware del variador.

        TODO: td_ref_17
        """
        try:
            return tag.attrib['firmware']
        except KeyError:
            return '[FIRMWARE] ERROR/NO DISPONIBLE'

    def _model_name(self):
        """Extrae el número de serie del variador.

        TODO: td_ref_18
        """
        try:
            for tag in self.root_val.iter('Model'):
                return tag.attrib['Name']
        except KeyError:
            return '[MODEL NAME] ERROR'

    def _producto(self):
        """Extrae el nombre del producto.

        TODO: td_ref_19
        """
        if self.root_val is None:
            logger.error(self.loc['err']['e7'])
            return None

        try:  # Utiliza ruta xml
            for p in self.root_val.findall("./drive/Classifier/Product"):
                return p.text
        except KeyError:
            return '[PRODUCT] ERROR'

    def _voltage(self):
        """Extrae el voltage del producto."""
        try:  # Contenido de <> <>
            for tag in self.root_val.iter('Voltage'):
                return tag.text
        except KeyError:
            return '[PRODUCT] VOLTAGE'

    def _current(self):
        """Extrae el voltage del producto."""
        try:  # Contenido de <> <>
            for p in self.root_val.iter('HDDriveRatedCurrentA'):
                return p.text
        except KeyError:
            return '[PRODUCT] CURRENT'

    def drive_header(self):
        """Retorna información sobre el variador.

        :return: The header information extracted from the Nidec parameter
        file, or None if the analysis fails.
        """
        if self.root_val is None:
            logger.error(self.loc['err']['e7'])
            return None

        # Verifica si self.root_val es iterable
        try:
            iter(self.root_val)
        except TypeError as e:
            logger.error(self.loc['err']['e7'], e)
            return None

        nidec_drive = NidecDrive()

        try:
            nidec_drive.param_file = self.param_file
        except AttributeError as e:
            logger.error(self.loc['err']['e9'], e)
            return None

        #
        # Proceso de ficheros
        #

        try:
            # Ruta XML
            nidec_drive.product = self._producto()

            self._scan_slots(nidec_drive)
            self._ip_node(nidec_drive)
            self._modo_trabajo(nidec_drive)
        except AttributeError as e:
            logger.error(self.loc['err']['e9'], e)
            return None

        # Contenido dentro de <
        try:
            for p in self.root_val.iter('drive'):
                nidec_drive.name = self._name(p)
                nidec_drive.serial_number = self._serial_number(p)
                nidec_drive.firmware = self._firmware(p)
        except AttributeError as e:
            logger.error(self.loc['err']['e9'], e)
            return None

        try:
            nidec_drive.model = self._model_name()

            # Contenido de <> <>
            nidec_drive.voltage = self._voltage()
            nidec_drive.current = self._current()
        except AttributeError as e:
            logger.error(self.loc['err']['e9'], e)
            return None

        return nidec_drive



"""Define un accionamiento NIDEC."""

from dataclasses import dataclass


@dataclass()
class NidecDrive:
    """Define un accionamiento Nidec.

    Atributos:
        - product (str): nombre del producto.
        - model (str): modelo del dispositivo.
        - serial_number (str): número de serie del dispositivo.
        - firmware (str): versión del firmware del dispositivo.
        - voltage (str): tensión nominal del dispositivo.
        - current (str): corriente nominal del dispositivo.
        - name (str): nombre asignado al dispositivo.
        - mode (str): modo de trabajo del dispositivo.
        - ip_nodo (str): ip o nodo (si emplea comunicación modbus).
        - slot1 (str): contenido del slot, si existe.
        - slot2 (str): contenido del slot, si existe.
        - slot3 (str): contenido del slot, si existe.
        - slot4 (str): contenido del slot, si existe.
        - param_file (str): path del fichero de parámetros.
    """

    product: str = ''
    model: str = ''
    serial_number: str = ''
    firmware: str = ''
    voltage: str = ''
    current: str = ''
    name: str = ''
    mode: str = ''
    ip_nodo: str = ''
    slot1: str = 'Not available'
    slot2: str = 'Not available'
    slot3: str = 'Not available'
    slot4: str = 'Not available'
    param_file: str = ''



from setuptools import setup, find_packages

setup(
    name="nidec-inventory",
    version="0.0a14",
    description="Nidec Unidrive M motor control drives inventory generator.",
    long_description=open('README.md',"r", encoding="utf-8").read(),
    long_description_content_type='text/markdown',
    author="Carlos Alonso Martín",
    author_email="cam.codeweaver@gmail.com",
    url="https://github.com/carlosalma/nidec-inventory",
    project_urls={"Github": "github.com/carlosalma/nidec-inventory",},
    license="MIT License",
    packages=find_packages(),
    package_data={'': ["*.yaml"]}
)
